@@ -29,7 +29,7 @@ import threading
 class AllInOne:
     def __init__(self, dirToParse):
         
-	self.rdr = CfgFileReader(dirToParse);
+	self.rdr = CfgFileReader(dirToParse+os.sep+'Server-mode');
 	self.rdr.run()
 
 class CfgFileReader(threading.Thread):
@@ -52,7 +52,12 @@ class CfgFileReader(threading.Thread):
 
 	    fillCMD='FILL_LIST=Set(iconlist,replace,Applications';
 	    
-	    files = os.listdir(self.cfgDir)
+	    try:
+	         files = os.listdir(self.cfgDir)
+	    except OSError:
+	         print "Error: directory not exists "+self.cfgDir
+		 return
+		 
 	    files.sort()
 	    i = 0;
 	    iconsList = []
@@ -114,13 +119,15 @@ class CfgFileReader(threading.Thread):
 	
 	    ex = 1
 	    if e != None:
-	        which = os.popen('which '+e).readlines()
+	        which = os.popen('which '+e+' 2> /dev/null').readlines()
 		if which == []:
+		    #print 'Skip ',e
 		    ex = 0
 	
 	    if a == 1 and i != None and ex == 1:
 	        ret = [i,n]   
 	
+	if fd:
 	    fd.close()
 	    
 	return ret
