@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,34 +18,36 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 //
- 
+
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <math.h>
+#include <netdb.h> 
+#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <termios.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/un.h>
-#include <fcntl.h>
-#include <sys/time.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/un.h>
+#include <termios.h>
 #include <unistd.h>
-#include <netdb.h> 
 
-//#define USE_BT
-
-#ifdef USE_BT
+#ifdef USE_BLUEZ
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/rfcomm.h>
 #endif
 
+#ifdef USE_BT_FBSD
+#include <bluetooth.h> 
+#include <sdp.h> 
+#include <err.h>
+#endif
+
 #include "btio.h"
 #include "common.h"
 #include "parse.h"
@@ -65,6 +67,8 @@ static struct termios portterm;
 
 char * uSocket = NULL;
 
+static char * btAddress = NULL;
+
 int bt_open_port(char* port) 
 {
 	
@@ -482,6 +486,150 @@ int unix_read_port(char* buf, int l)
 	return ret;
 }
 */
+
+
+//
+// Support SDP
+//
+
+#ifdef USE_BLUEZ
+ 
+#include <bluetooth/sdp.h>
+#include <bluetooth/sdp_lib.h>
+
+sdp_session_t *session = NULL;
+sdp_record_t  *record  = NULL; 
+
+void sdpRegister(int port)
+{
+    uint8_t svc_uuid_int[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xab, 0xcd };
+    uint8_t rfcomm_channel = port;
+    const char *svc_dsc = "Bluetooth remote control";
+    const char *service_prov = "anyRemote";
+
+    uuid_t root_uuid, l2cap_uuid, rfcomm_uuid, svc_uuid, 
+           svc_class_uuid;
+    sdp_list_t *l2cap_list = 0, 
+               *rfcomm_list = 0,
+               *root_list = 0,
+               *proto_list = 0, 
+               *access_proto_list = 0,
+               *svc_class_list = 0,
+               *profile_list = 0;
+    sdp_data_t *channel = 0;
+    sdp_profile_desc_t profile;
+    record = sdp_record_alloc();
+
+    // set the general service ID
+    sdp_uuid128_create( &svc_uuid, &svc_uuid_int );
+    sdp_set_service_id( record, svc_uuid );
+
+    // set the service class
+    sdp_uuid16_create(&svc_class_uuid, SERIAL_PORT_SVCLASS_ID);
+    svc_class_list = sdp_list_append(0, &svc_class_uuid);
+    sdp_set_service_classes(record, svc_class_list);
+
+    // set the Bluetooth profile information
+    sdp_uuid16_create(&profile.uuid, SERIAL_PORT_PROFILE_ID);
+    profile.version = 0x0100;
+    profile_list = sdp_list_append(0, &profile);
+    sdp_set_profile_descs(record, profile_list);
+
+    // make the service record publicly browsable
+    sdp_uuid16_create(&root_uuid, PUBLIC_BROWSE_GROUP);
+    root_list = sdp_list_append(0, &root_uuid);
+    sdp_set_browse_groups( record, root_list );
+
+    // set l2cap information
+    sdp_uuid16_create(&l2cap_uuid, L2CAP_UUID);
+    l2cap_list = sdp_list_append( 0, &l2cap_uuid );
+    proto_list = sdp_list_append( 0, l2cap_list );
+
+    // register the RFCOMM channel for RFCOMM sockets
+    sdp_uuid16_create(&rfcomm_uuid, RFCOMM_UUID);
+    channel = sdp_data_alloc(SDP_UINT8, &rfcomm_channel);
+    rfcomm_list = sdp_list_append( 0, &rfcomm_uuid );
+    sdp_list_append( rfcomm_list, channel );
+    sdp_list_append( proto_list, rfcomm_list );
+
+    access_proto_list = sdp_list_append( 0, proto_list );
+    sdp_set_access_protos( record, access_proto_list );
+
+    // set the name, provider, and description
+    sdp_set_info_attr(record, getServiceName(), service_prov, svc_dsc);
+
+    // connect to the local SDP server, register the service record, 
+    // and disconnect
+    session = sdp_connect(BDADDR_ANY, BDADDR_LOCAL, SDP_RETRY_IF_BUSY);
+    if ( (!(session && record)) || sdp_record_register(session, record, 0) == -1) {
+        logger("ERROR", "can not register SDP service");
+    }
+
+    // cleanup
+    sdp_data_free( channel );
+    sdp_list_free( l2cap_list, 0 );
+    sdp_list_free( proto_list, 0 );
+    sdp_list_free( rfcomm_list, 0 );
+    sdp_list_free( root_list, 0 );
+    sdp_list_free( access_proto_list, 0 );
+    sdp_list_free( svc_class_list, 0 );
+    sdp_list_free( profile_list, 0 );
+}
+#endif
+
+#ifdef USE_BT_FBSD
+void			*session = NULL;
+uint32_t		 record;
+
+
+void sdpRegister(int port)
+{
+	int channel,service;
+	bdaddr_t		 bt_addr_any;
+	sdp_lan_profile_t	 lan;
+	
+	channel = port;
+	service = SDP_SERVICE_CLASS_SERIAL_PORT;	
+	
+	session = sdp_open_local(NULL);
+	if (session == NULL)
+		errx(1, "Unable to create local SDP session");
+	if (sdp_error(session) != 0)
+	errx(1, "Unable to open local SDP session. %s (%d)",
+		strerror(sdp_error(session)), sdp_error(session));
+	memset(&lan, 0, sizeof(lan));
+	lan.server_channel = channel;
+	
+	memcpy(&bt_addr_any, NG_HCI_BDADDR_ANY, sizeof(bt_addr_any));
+	if (sdp_register_service(session, service, &bt_addr_any,
+		(void *)&lan, sizeof(lan), &record) != 0) {
+	errx(1, "Unable to register LAN service with "
+		"local SDP daemon. %s (%d)",
+		strerror(sdp_error(session)), sdp_error(session));
+		}
+}
+#endif
+
+void sdpDeregister()
+{
+    #ifdef USE_BLUEZ
+    if (session != NULL) {
+        sdp_record_unregister(session, record);
+	sdp_close(session);
+        session = NULL;
+        //sdp_record_free(record);
+    }
+    #endif
+    #ifdef USE_BT_FBSD
+    if (session != NULL) {
+	    sdp_unregister_service(session, record);
+	    sdp_close(session);
+	    sdp_close(session);
+	    session = NULL;
+    }
+    #endif
+}
+
 //
 // Support server mode sockets
 //
@@ -490,14 +638,18 @@ int openSocketPort(int type, int port, char *path)
 {
 	struct sockaddr_in tcp_addr;
 	struct sockaddr_un un_addr;
-	#ifdef USE_BT
+	#ifdef USE_BLUEZ
 	struct sockaddr_rc bt_addr;
 	#endif
+	#ifdef USE_BT_FBSD
+	struct sockaddr_rfcomm bt_addr;
+	#endif
 	
 	struct sockaddr*   socketaddr = NULL;
 
 	int addFamily = 0;
 	int proto     = 0;
+	int sz;
 
 	if (sportfd) { 
 		logger("ERROR", "socket was already opened");
@@ -510,10 +662,14 @@ int openSocketPort(int type, int port, char *path)
 		addFamily = AF_INET;
 		proto	  = IPPROTO_TCP;
 	} else if (type == SERVER_BT) {
-		#ifdef USE_BT
+		#ifdef USE_BLUEZ
 		addFamily = PF_BLUETOOTH;
 		proto	  = BTPROTO_RFCOMM;
 		#endif
+		#ifdef USE_BT_FBSD
+		addFamily = PF_BLUETOOTH;
+		proto	  = BLUETOOTH_PROTO_RFCOMM;
+		#endif
 	} else if (type == SERVER_UX) {
 		addFamily = AF_UNIX;
 		proto	  = 0;
@@ -549,26 +705,43 @@ int openSocketPort(int type, int port, char *path)
 		tcp_addr.sin_port = htons(port);
 	
 		socketaddr=(struct sockaddr *)&tcp_addr;
+		sz = sizeof(tcp_addr);
 		
 	} else if (type == SERVER_BT){	// SERVER_BT
-		#ifdef USE_BT
+		#ifdef USE_BLUEZ
 		memset((void *) &bt_addr, 0, sizeof(bt_addr));
+		sz = sizeof(bt_addr);
 		
 		// bind socket to the specified port of the first available local bluetooth adapter
 		bt_addr.rc_family = AF_BLUETOOTH;
 		bt_addr.rc_bdaddr = *BDADDR_ANY; 
 		bt_addr.rc_channel = (uint8_t) port;
 
-		char tmpstring[512];
+		//char tmpstring[512];
 		//sprintf(tmpstring, "sdptool add --channel=%i SP", port);
 		//sprintf(tmpstring, "sdptool add --channel=%i SP;sdptool setattr `sdptool search --bdaddr local SP|grep \"Service RecHandle\"|tail -1|cut -f 3 -d \" \"` 0x100 anyRemote", port);
-		sprintf(tmpstring, "bash -c \'A=`sdptool search --bdaddr local SP|grep \"Service Name\"|grep anyRemote|wc -l`; if [ \"x$A\" == \"x0\" ]; then sdptool add --channel=%i SP;sdptool setattr `sdptool search --bdaddr local SP|grep \"Service RecHandle\"|tail -1|cut -f 3 -d \" \"` 0x100 anyRemote; fi\'", port);
-		system(tmpstring);
+		//sprintf(tmpstring, "bash -c \'A=`sdptool search --bdaddr local SP|grep \"Service Name\"|grep anyRemote|wc -l`; if [ \"x$A\" == \"x0\" ]; then sdptool add --channel=%i SP;sdptool setattr `sdptool search --bdaddr local SP|grep \"Service RecHandle\"|tail -1|cut -f 3 -d \" \"` 0x100 anyRemote; fi\'", port);
+		//system(tmpstring);
+
+                sdpRegister(port);
 		sprintf(tmp, "registered SP for channel %i", port);
 		logger("INFO", tmp);
-
 		socketaddr=(struct sockaddr *)&bt_addr;
 		#endif
+		
+		#ifdef USE_BT_FBSD
+		memset(&bt_addr, 0, sizeof(bt_addr));
+
+		bt_addr.rfcomm_len = sizeof(bt_addr);
+		bt_addr.rfcomm_family = AF_BLUETOOTH;
+		bt_addr.rfcomm_channel =  (uint8_t) port;;
+		sdpRegister(port);
+		sprintf(tmp, "registered SP for channel %i", port);
+		logger("INFO", tmp);
+		
+		socketaddr=(struct sockaddr *)&bt_addr;
+		sz = sizeof(bt_addr);
+		#endif
 	} else if (type == SERVER_UX && path != NULL){
 	
 		memset(&un_addr, 0, sizeof(un_addr));
@@ -576,16 +749,16 @@ int openSocketPort(int type, int port, char *path)
 		strncpy(un_addr.sun_path, path, sizeof un_addr.sun_path - 1);
 		printf("ERROR: SOCKET %s\n", path);
 		socketaddr=(struct sockaddr *)&un_addr;
+		sz = sizeof(un_addr);
 	
 	} else {
 		logger("ERROR", "incorrect input 2");
 		return -1;
 	}
 
-	if (bind(sportfd, (struct sockaddr *) socketaddr, sizeof(*socketaddr)) < 0) {
+	if (bind(sportfd, (struct sockaddr *) socketaddr, sz) < 0) {
 		logger("ERROR", "on binding");
 		printf("ERROR: on binding %d->%s\n", errno, strerror(errno));
-	
 		return -1;
 	}
 	
@@ -611,7 +784,7 @@ int closeSocketPort(int type)
 	if (type == SERVER_UX) {
 		unlink(uSocket);
 		free(uSocket);
-	}
+        }
 	
 	return 1;
 }
@@ -623,16 +796,20 @@ int listenAndAcceptSocketConn(int type)
 	struct sockaddr* socketaddr = NULL;
 	struct sockaddr_in ip_addr;
 
-	#ifdef USE_BT
+	#ifdef USE_BLUEZ
 	struct sockaddr_rc bt_addr;
 	bdaddr_t ba;
 	#endif
 
+	#ifdef USE_BT_FBSD
+	struct sockaddr_rfcomm bt_addr;
+	#endif
+
 	logger("INFO", "listenAndAcceptSocketConn");
 	cnt = 0;
 
 	if (type == SERVER_BT) {
-		#ifdef USE_BT
+		#if defined(USE_BLUEZ) || defined(USE_BT_FBSD)
 		socketaddr=(struct sockaddr *)&bt_addr;
 		sz = sizeof(bt_addr);
 		#endif
@@ -674,13 +851,27 @@ int listenAndAcceptSocketConn(int type)
 		*/
 		}
 		logger("INFO", "listenAndAcceptSocketConn: accepted");
-
 	
-		#ifdef USE_BT
+		#ifdef USE_BLUEZ
 		if (type == SERVER_BT) {
 			baswap(&ba, &bt_addr.rc_bdaddr);
-
-			sprintf(tmp, "listenAndAcceptSocketConn: remote BT address is %s", batostr(&ba));
+			if (btAddress) {
+				free(btAddress);
+			}
+			btAddress = batostr(&ba);
+			
+			sprintf(tmp, "listenAndAcceptSocketConn: remote BT address is %s", btAddress);
+			logger("INFO", tmp);
+		}
+		#endif
+		#ifdef USE_BT_FBSD
+		if (type == SERVER_BT) {
+			if (btAddress) {
+				free(btAddress);
+			}
+			btAddress = strdup(bt_ntoa(&bt_addr.rfcomm_bdaddr, NULL));
+			
+			sprintf(tmp, "listenAndAcceptSocketConn: remote BT address is %s", btAddress);
 			logger("INFO", tmp);
 		}
 		#endif
@@ -879,3 +1070,14 @@ int readFromFrontEnd(char *buf, int size)
 	
 	return EXIT_OK;
 }
+
+char * getBtAddress()
+{
+	return btAddress;
+}
+
+void freeBtAddress() 
+{
+	if (btAddress) free(btAddress); 
+	btAddress = NULL;
+}
