@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,9 +37,15 @@ int  bt_put_command(char* command,
 
 int  bt_getfd(void);
 
+char * getBtAddress(void);
+void freeBtAddress (void);
+
 int closeSocketPort(int type);
 int openSocketPort(int type, int port,char *path);
 
+// SDP 
+void sdpDeregister();
+
 void openStdin();
 void closeStdin();
 int readStdin(char* buf, int max);
