@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -141,9 +141,8 @@ static char *readResultsFile(int * count)
 
 static void freeResults(char * ptr) 
 {
-	logger("DEBUG","freeResults");
+	//logger("DEBUG","freeResults");
 	if (ptr != resBuffer) {
-        	logger("DEBUG","freeResults: free");
         	free(ptr);
         }
 }
@@ -169,6 +168,8 @@ static char* substParams(const char* in, cmdParams *params) {
         int pp = strlen(PARAM_PARAM);
         int pi = strlen(PARAM_INDEX);
         int pd = strlen(PARAM_CFGDIR);
+        int ph = strlen(PARAM_HOME);
+        int pb = strlen(PARAM_BTADDR);
         
         // calculate size to allocate
         int needSize = strlen(ptr)+1; 
@@ -208,6 +209,24 @@ static char* substParams(const char* in, cmdParams *params) {
                         }
                         ptr = papameterized + pd + 2;
                         nParams++;
+                } else if (strncmp(papameterized + 2,PARAM_HOME,ph) == 0) {
+			int c = 0;
+                        char *d;
+                        if ((d = getenv("HOME")) != NULL) {
+				c = strlen(d);
+                                needSize += c - ph - 2;
+                        }
+                        ptr = papameterized + ph + 2;
+                        nParams++;
+                } else if (strncmp(papameterized + 2,PARAM_BTADDR,pb) == 0) {
+			int c = 0;
+                        char *d;
+                        if ((d = getBtAddress()) != NULL) {
+				c = strlen(d);
+                                needSize += c - pb - 2;
+                        }
+                        ptr = papameterized + pb + 2;
+                        nParams++;
                 } else {
                 	int pLen = (papameterized == NULL ? 0 : 1);
                         if (pLen) {
@@ -339,6 +358,40 @@ static char* substParams(const char* in, cmdParams *params) {
                 
                         ptr = papameterized + pd + 2;
                 
+                } else if (strncmp(papameterized + 2,PARAM_HOME,ph) == 0) {
+                
+                        *papameterized = '\0';
+                
+                        // First part
+                        strcpy(cmd+count, ptr);
+                	count += strlen(ptr);
+
+                        // Add $HOME
+                        char *d;
+                        if ((d = getenv("HOME")) != NULL) {
+				strcpy(cmd+count, d);
+                        	count += strlen(d);
+                        }
+                
+                        ptr = papameterized + ph + 2;
+
+                } else if (strncmp(papameterized + 2,PARAM_BTADDR,pb) == 0) {
+                
+                        *papameterized = '\0';
+                
+                        // First part
+                        strcpy(cmd+count, ptr);
+                	count += strlen(ptr);
+
+                        // Add XX:XX:XX:XX:XX:XX
+                        char *d;
+                        if ((d = getBtAddress()) != NULL) {
+				strcpy(cmd+count, d);
+                        	count += strlen(d);
+                        }
+                
+                        ptr = papameterized + pb + 2;
+                
                 } else { 
 
                 	int pLen = (papameterized == NULL ? 0 : 1);
@@ -497,29 +550,33 @@ int execDynamically(char *command)
 
 static int loadCmds(char *file) 
 {
-	//logger("DEBUG","loadCmds");
+	logger("DEBUG","loadCmds");
         if (file == NULL) {
         	return EXIT_NOK;
         }
 
-	FILE *fp;
-        char *command = NULL;
-        size_t len = 0;
-        ssize_t read;
-	
-	fp=fopen(file,"r");
+	FILE *fp=fopen(file,"r");
 	if (fp == NULL) {
 		return EXIT_NOK;
 	}
-        
+
+        struct stat file_status;
+	stat(file, &file_status);
+	
+	if (file_status.st_size <= 0) {
+		logger("DEBUG","loadCmds - file is empty");
+		return EXIT_NOK;
+	}
+	
+	char *command = malloc(file_status.st_size+1);
+	*command = '\0';
+	
 	int ret = EXIT_OK;
-	while ((read = getline(&command, &len, fp)) != -1) {
-        	DEBUG2("Retrieved line %s of length %d ", command, (int) read);
+	while (fgets(command, file_status.st_size+1, fp) != NULL) {
+        	DEBUG2("Retrieved line %s of length %d ", command, strlen(command));
 		ret = execDynamically(command); 	
         }
-        if (command) {
-        	free(command);
-        }
+        free(command);
         fclose(fp);
 	return ret;
 }
@@ -640,11 +697,13 @@ static int setModeCmd(const char *sDescr)
 
 static int getCmd(const char *sDescr) 
 {
-	if (sDescr != NULL &&
-            (serverMode == SERVER_BT  || 
-             serverMode == SERVER_TCP || 
-             serverMode == SERVER_UX  || 
-             serverMode == CLIENT_NOAT)) {
+	if (sDescr == NULL) return EXIT_NOK;
+	
+	
+        if (serverMode == SERVER_BT  ||
+            serverMode == SERVER_TCP || 
+            serverMode == SERVER_UX  || 
+            serverMode == CLIENT_NOAT) {
              
         	strcpy(tmptext, CMD_GET);
         	strcat(tmptext, "(");
@@ -656,7 +715,22 @@ static int getCmd(const char *sDescr)
          		logger("DEBUG","Fails in writeSocketConn() (Get)");
          	}
                 return ret;
-        }    
+        }  
+	#ifdef USE_L2CAP  
+        if (serverMode == SERVER_L2CAP) {
+             
+        	strcpy(tmptext, CMD_GET);
+        	strcat(tmptext, "(");
+        	strcat(tmptext, sDescr);
+        	strcat(tmptext, ");");
+
+        	int ret = writeL2capConn(tmptext, strlen(tmptext));
+        	if (ret != EXIT_OK) {
+         		logger("DEBUG","Fails in writeL2capConn() (Get)");
+         	}
+                return ret;
+        } 
+	#endif   
 	return  EXIT_OK;
 }
 
@@ -727,7 +801,12 @@ static int makeCmd(cmdItem* ci, cmdParams* p)
         } else if (strstr(ci->descr,MAKE_MODE) == ci->descr) {
         
 		logger("INFO", "Command: Make(mode,...)");
-                ret = setModeCmd(ci->exec);                
+                
+		char* sExec = substParams(ci->exec, p);
+                
+                ret = setModeCmd(sExec); 
+		
+		free(sExec); sExec = NULL;               
         
         } else if (strstr(ci->descr,MAKE_REMOTE) == ci->descr) {
         
@@ -916,6 +995,9 @@ int macroCmd(char *descr, char *exec, cmdParams* params)
                         }                   	
                 } else if (exec && strstr(exec,"if_mode_server") == exec) {
 			if (serverMode != SERVER_BT  &&
+			    #ifdef USE_L2CAP
+                            serverMode != SERVER_L2CAP &&
+			    #endif
                             serverMode != SERVER_TCP &&
                             serverMode != SERVER_UX  &&
                             serverMode != CLIENT_NOAT) {
@@ -1107,7 +1189,10 @@ static int processOneCommand(cmdItem* ci, cmdParams* p)
 
             	case ID_EXECSET: 
                		logger("INFO", "Command: ExecAndSet");
-                	if (serverMode == SERVER_BT  || 
+                	if (serverMode == SERVER_BT  ||
+			    #ifdef USE_L2CAP 
+                            serverMode == SERVER_L2CAP || 
+			    #endif
                             serverMode == SERVER_TCP || 
                             serverMode == SERVER_UX  || 
                             serverMode == CLIENT_NOAT) { 
@@ -1119,6 +1204,9 @@ static int processOneCommand(cmdItem* ci, cmdParams* p)
             	case ID_EXECSEND:  
                 	logger("INFO", "Command: ExecAndSend");
                 	if (serverMode == SERVER_BT  || 
+ 			    #ifdef USE_L2CAP 
+                            serverMode == SERVER_L2CAP || 
+			    #endif
                             serverMode == SERVER_TCP || 
                             serverMode == SERVER_UX  || 
                             serverMode == CLIENT_NOAT) {
@@ -1174,6 +1262,9 @@ static int processOneCommand(cmdItem* ci, cmdParams* p)
             	case ID_SEND:
                 	logger("INFO", "Command: Send");
                 	if (serverMode == SERVER_BT  || 
+			    #ifdef USE_L2CAP 
+                            serverMode == SERVER_L2CAP || 
+			    #endif
                             serverMode == SERVER_TCP || 
                             serverMode == SERVER_UX  ||  
                             serverMode == CLIENT_NOAT) {
@@ -1185,6 +1276,9 @@ static int processOneCommand(cmdItem* ci, cmdParams* p)
                 	logger("INFO", "Command: Set");
                 	if (sDescr != NULL &&
 			    (serverMode == SERVER_BT   || 
+			     #ifdef USE_L2CAP 
+                             serverMode == SERVER_L2CAP || 
+			     #endif
 			     serverMode == SERVER_TCP  || 
 			     serverMode == SERVER_UX   || 
 			     serverMode == CLIENT_NOAT
