@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,6 +30,8 @@
 #define PARAM_PARAM     "Param)"
 #define PARAM_INDEX     "Index)"
 #define PARAM_CFGDIR    "CfgDir)"
+#define PARAM_HOME      "Home)"
+#define PARAM_BTADDR    "BtAddr)"
 
 #define SEND_STRING     "string"
 #define SEND_BYTES      "bytes"
