@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,11 +35,11 @@
   
 #define IS_OK(answer) (strstr(answer, "OK") != NULL)
 
-#define AR_VERSION 	"v4.6"
-
 #define AT_DEVICE   	"at:"
 #define INET_SOCKET   	"socket:"
 #define BT_SOCKET   	"bluetooth"
+#define BT_SPP   	"btspp" 	// not yet used
+#define L2CAP_SOCKET   	"l2cap"
 #define UNIX_SOCKET   	"local:"
 #define ILIRC_SOCKET   	"ilirc:"
 #define STDIN_STREAM   	"stdin"
@@ -53,7 +53,14 @@
 #define SERVER_UX       4
 #define CLIENT_NOAT     5
 #define SERVER_STDIN    6
+#define SERVER_L2CAP    7
 
 #define DEFAULT_BT_CHANNEL 19
+#define DEFAULT_L2CAP_PORT 5019
+
+// TODO: Support L2CAP under FreeBSD
+//#ifdef USE_BLUEZ
+//#define USE_L2CAP 1
+//#endif
 
 #endif
