@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ extern type_key* findExact(mode *mode, char *key);
 
 extern mode       *modes;
 extern mode       *currentMode;
+extern mode       *internalMode;
 extern mode 	  *defMode;
 extern type_alias *aliases;
 extern type_key   *alarms;
@@ -106,6 +107,10 @@ char* getToMainMenu()
 {
 	return  (conf.toMain[0] == '\0' ? NULL : conf.toMain);
 }
+char* getServiceName()
+{
+	return conf.serviceName;
+}
 
 char* getAT_CMER(int what)
 {
@@ -241,7 +246,6 @@ static type_key* findStartingWith(mode *mode, char *key)
 	return It;
 }
 
-
 static type_key* findItemInMode(mode *mode, char *key, int *flag, cmdParams *params) 
 {
 	if (mode == NULL) {
@@ -398,6 +402,16 @@ type_key* findItem(char *keyIn, int *flag, cmdParams *params)
 			tk = findItemInMode(defMode, key, flag, params);
 		}
 	}
+	
+	//  Search in "internal" commands
+	if (tk == NULL) {
+	        logger("DEBUG","findItem() internal mode");
+		tk = findItemInMode(internalMode, key, flag, params);
+		if (tk != NULL && *flag == FLAG_MULTIKEY) {
+			logger("DEBUG","findItem() multikey in internal command ?");
+		}
+	}
+	
 	//logger("DEBUG","findItem() EXIT");
         /*if (tk && tk->cmd) {
         	printf("TRACE %s->%s:%s \n",keyIn,tk->cmd->descr,(tk->cmd->exec ? tk->cmd->exec : "NULL"));
@@ -540,7 +554,11 @@ void printParams()
 {
 	int atMode = 1;
 	if (strstr(conf.device, INET_SOCKET) != NULL || 
-	    strstr(conf.device, BT_SOCKET) != NULL) {
+	    strstr(conf.device, BT_SOCKET)   != NULL
+	    #ifdef USE_L2CAP
+	    || strstr(conf.device, L2CAP_SOCKET) != NULL
+	    #endif
+	    ) {
 		atMode = 0;
 	}
 
@@ -587,7 +605,8 @@ void printParams()
 
 void printConf() 
 {
-        sprintf(tmp, "anyRemote %s", AR_VERSION);
+	printTime();
+        sprintf(tmp, "anyRemote v%s", PACKAGE_VERSION);
         logger("CFG", tmp);
 	printKeys   ();
 	printAliases();
@@ -632,6 +651,7 @@ void freeCfgEx(mode* pmodes, type_key* palarms, type_alias *paliases)
 	// Keys
 	while(pmodes) {
 		mode	 *md = pmodes;
+		//printf("mode -> %s\n",md->name);
 		type_key *mk = md->keys;
 	
 		while (mk) {
@@ -647,7 +667,7 @@ void freeCfgEx(mode* pmodes, type_key* palarms, type_alias *paliases)
 		pmodes = (mode*) pmodes->next;
 		free(md);
 	}
-
+	        
 	// Alarms
         type_key *mk = palarms;
 	while (mk) {
@@ -664,6 +684,8 @@ void freeCfgEx(mode* pmodes, type_key* palarms, type_alias *paliases)
 	// Aliases
 	while(paliases) {
 		type_alias *al = paliases->next;
+		free(paliases->key);
+		free(paliases->alias);		
 		free(paliases); paliases = NULL;
 		paliases = al;
 	}
@@ -672,5 +694,23 @@ void freeCfgEx(mode* pmodes, type_key* palarms, type_alias *paliases)
 void freeCfg(void)
 {
 	freeCfgEx(modes, alarms, aliases);
+        
+        // special handling for "internal" mode
+        if (internalMode != NULL) {
+		type_key *mki = internalMode->keys;
+		while (mki) {
+			type_key* It = mki;
+			mki = (type_key*) mki->next;
+		
+                	freeCmds(It->cmd);
+ 
+ 			It->cmd = NULL;
+                	free(It->key);
+			free(It);
+		}
+		free(internalMode);
+		internalMode = NULL;
+        }
+        
         free(conf.cfgDir);
 }
