@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,6 +32,10 @@
 #include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
+#ifdef __FreeBSD__
+	#include <signal.h>
+#endif
+
 
 #include "atsend.h"
 #include "btio.h"
@@ -42,12 +46,19 @@
 #include "utils.h"
 #include "xemulate.h"
 
+#ifdef USE_L2CAP
+	#include "pr_l2cap.h"
+#endif
+
 #define TMPDISCONN_TIME 60
 #define ALARM_CLEAN 	0
 #define ALARM_CHECK 	1
 
+extern CONF       conf;
+
 static void mainRoutine     (void);
 static int  isBemusedCommand(char *cmd);
+static int  isInpLircCommand(char *cmd);
 static int  isLircCommand   (char *cmd);
 
 char tmp	    [MAXMAXLEN];
@@ -161,6 +172,25 @@ int openPort(char * portIn)
                 	return -1;
                 }
                 serverMode = SERVER_BT;
+		
+       #ifdef USE_L2CAP
+       } else if ((strPort = strstr(portIn, L2CAP_SOCKET)) != NULL) {
+
+        	strPort += strlen(L2CAP_SOCKET);
+        
+                if (strstr(strPort, ":") == NULL) { // just "l2cap"
+                	port = DEFAULT_L2CAP_PORT;
+                } else {
+                	strPort++;
+                        port = atoi(strPort);
+                }
+                DEBUG2("L2CAP Server mode. Use port %d", port);
+
+                if ((ret = openL2capPort(port)) < 0) {
+                	return -1;
+                }
+                serverMode = SERVER_L2CAP;
+        #endif
 
         } else if ((strPort = strstr(portIn, UNIX_SOCKET)) != NULL){
 
@@ -200,6 +230,13 @@ int openPort(char * portIn)
                 }
                 initialized++;
         }
+	
+	// setgid
+	if(conf.uid && getuid()==0){
+		DEBUG2("setuid/setgid %d,%d",conf.uid,conf.gid);
+		setgid(conf.gid);	
+		setuid(conf.uid);
+	}
         return 1;
 }
 
@@ -215,6 +252,9 @@ void createDataDir()
         	closedir(d);
         } else {
 		mkdir(dd, S_IRWXU);
+		if(getuid()==0 && conf.uid) { // do not create as superuser
+			chown(dd,conf.uid,conf.gid);
+		}
 	}
 }
 
@@ -225,7 +265,11 @@ void closePort(int final)
             serverMode == SERVER_TCP || 
             serverMode == SERVER_UX) {
          	closeSocketPort(serverMode);
-        } else if (serverMode == CLIENT_ILIRC) {
+	#ifdef USE_L2CAP	
+        } else if (serverMode == SERVER_L2CAP) {
+                closeL2capPort();
+	#endif
+	} else if (serverMode == CLIENT_ILIRC) {
         	if (unix_close_port() < 0) {
                 	printf("Error on closing AF_UNIX socket\n");
                 }
@@ -255,11 +299,22 @@ static void freeAndExit(void)
 		handleCmdByKey(k,NULL);   // Since we exiting now we did not interested in return code
 	}
 	
+        if (serverMode == SERVER_BT) {
+		sdpDeregister();
+        	freeBtAddress();
+	}
+	#ifdef USE_L2CAP
+        if (serverMode == SERVER_L2CAP) {
+		sdpDeregisterL2cap();
+	}
+	#endif
+	
 	freeDisplay();
 	freeTimers();
 	freeVars();
 	freeCachedData();
 	freeCfg();
+	freeRegexps();
 	exit(0);
 }
 
@@ -272,6 +327,11 @@ void sig_handle() {
 	    
 		writeSocketConn("Set(disconnect);", 16);
 		gotSignal = 1;
+	#ifdef USE_L2CAP	
+	} else if (serverMode == SERVER_L2CAP) {
+		writeL2capConn("Set(disconnect);", 16);
+		gotSignal = 1;
+	#endif
 	} else {	// CLIENT_AT, CLIENT_NOAT, SERVER_STDIN
 		freeAndExit();
 	}
@@ -305,15 +365,16 @@ int main(int argc, char *argv[])
                 }
         }
 	
-        createDataDir();
-	
         if (argv[1] && (strcmp(argv[1],"-h")==0 || strcmp(argv[1],"--help")==0)) {
 		 //Just print help and exit
 		 printHelp();
 	         exit(0);
         } else if (argv[1] && (strcmp(argv[1],"-v")==0 || strcmp(argv[1],"--version")==0)) {
-		printf("anyremote %s\n", AR_VERSION); 
+		printf("anyremote v%s\n", PACKAGE_VERSION); 
 		exit(0);
+        //} else if (argv[1] && strcmp(argv[1],"-getconfdir")==0) {
+	//	 printCfgDir();
+	//         exit(0);
         }
 
         if (init_cfg(i<argc ? argv[++i] : NULL) != EXIT_OK) {
@@ -330,7 +391,8 @@ int main(int argc, char *argv[])
                 freeCfg();
                 exit(1);
         }
-        
+
+        createDataDir();
         initLog();
         
         // Not to core dump if connection will close
@@ -631,6 +693,8 @@ int handle_command(char* cmd)
 		}
 	} else if (isLircCommand(cmd)) {	    // LIRC tricks
 		return EXIT_OK;
+	} else if (isInpLircCommand(cmd)) {	    // inputLIRC tricks
+		return EXIT_OK;
 	} else if (isBemusedCommand(cmd)) {	    // Bemused server emulation tricks
 		return EXIT_OK;
 	} else if (serverMode == SERVER_STDIN) {    // Just direct input, try it
@@ -754,9 +818,16 @@ int initPort()
 	    serverMode == SERVER_TCP || 
 	    serverMode == SERVER_UX) {	// Wait for incoming connection
                 logger("INFO", "Server mode: Waiting connection");
-                return listenAndAcceptSocketConn();
+                return listenAndAcceptSocketConn(serverMode);
         } 
 
+	#ifdef USE_L2CAP	
+        if (serverMode == SERVER_L2CAP) {
+                logger("INFO", "Server mode/L2CAP: Waiting connection");
+                return listenAndAcceptL2capConn();
+	} 
+	#endif
+
 	if (serverMode == CLIENT_ILIRC || 
 	    serverMode == CLIENT_NOAT  ||
 	    serverMode == SERVER_STDIN) {
@@ -900,6 +971,7 @@ static void mainRoutine(void)
 
 					// cleanup cached data
 					freeCachedData();
+					freeBtAddress();
 					
 					i = 0;
 	    				k = findItem(EVT_DISCONNECT, &i, NULL);
@@ -936,7 +1008,7 @@ static void mainRoutine(void)
 	    		
 	    				break;
 		    		} else if (retRead > 0) {
-					DEBUG2("-------------------- Command read -------------------- >%s<", answer);
+					DEBUG2("-------------------- Command read --------------------");
 	    	    
 		    			if (serverMode == CLIENT_NOAT) {	// in IR communication a lot of 'empty' chars (= -1) could be added
 						char * k2 = answer;
@@ -1070,14 +1142,20 @@ static void mainRoutine(void)
 		// Can't open port or it closed again
 		if (dmn || 
 		    serverMode == SERVER_BT   || 
+		    #ifdef USE_L2CAP 
+		    serverMode == SERVER_L2CAP || 
+		    #endif
 		    serverMode == SERVER_TCP  ||
 		    serverMode == SERVER_UX   || 
-		    serverMode == CLIENT_NOAT || 
+		    serverMode == CLIENT_NOAT ||
 		    ret == EXIT_DISCON) {
 		    
 	    		int timeout;
 	    	
-	    		if (serverMode == SERVER_BT  || 
+	    		if (serverMode == SERVER_BT  ||  
+			    #ifdef USE_L2CAP 
+		            serverMode == SERVER_L2CAP ||
+			    #endif
 		            serverMode == SERVER_TCP ||
 		            serverMode == SERVER_UX  || 
 		            serverMode == CLIENT_NOAT) {
@@ -1253,8 +1331,47 @@ static int isBemusedCommand(char *cmdIn)
 //
 // LIRC dev/event handling
 //
-      
+
 static int isLircCommand(char *cmdIn)
+{
+	//  should got replay in form "0000000000010184 00 TEXT linux-input-layer"
+	//                            "0000000000010192 00 CHANNELUP linux-input-layer"
+	DEBUG2("isLircCommand: >%s<",cmdIn);
+
+        char *cmd = strdup(cmdIn);
+        char *bufPtr = NULL;
+	
+	if (strncmp(cmdIn, "000000",6) != 0) {
+		return 0;
+	}
+	
+        char *token = strtok_r(cmd," ",&bufPtr);
+	if (token == NULL) {
+		free(cmd);
+		return 0;
+	}
+
+	token = strtok_r(NULL," ",&bufPtr);
+	if (token == NULL) {
+		free(cmd);
+		return 0;
+	} 
+	
+	token = strtok_r(NULL," ",&bufPtr);
+	if (token == NULL) {
+		free(cmd);
+		return 0;
+	} 
+	handle_key_press(token,1);
+
+	free(cmd);
+        return 1;
+}  
+//
+// inputlirc dev/event handling
+//
+
+static int isInpLircCommand(char *cmdIn)
 {  
 	// should got replay in form "2 0 KEY_1 event6"
 	
@@ -1269,12 +1386,12 @@ static int isLircCommand(char *cmdIn)
 
 	char *data = NULL;
         char *cmd = strdup(cmdIn);
-        char *bufPtr;
+        char *bufPtr = NULL;
         char *token = strtok_r(cmd," ",&bufPtr);
 	if (token == NULL) {
 		free(cmd);
 		return 0;
-	} 
+	}
 	
 	token = strtok_r(NULL," ",&bufPtr);
 	if (token == NULL) {
