@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <termios.h>
+#include <sys/stat.h>
 
 #include "parse.h"
 #include "utils.h"
@@ -43,16 +44,16 @@
 #include <regex.h>
 
 #define REGEX_KEYVAL   "^[[:space:]]*(\\[[[:alpha:]]+\\])([^=]*)$|^[[:space:]]*([^=%]*[^[:space:]]{1})[[:space:]]*=[[:space:]]*(.*[^[:space:]]{1})[[:space:]]*$"
-#define REGEX_CMDBYCMD "[[:space:]]*(Exec|ExecAndSet|Make|SetMode|Get|Set|Send|ExecAndSend|Timer|CancelTimer|Macro|Load|Include|SendCKPD|Emulate)[[:space:]]*(\\()|[[:space:]]*(RemoteOn|RemoteOff|Exit|TempDisconnect)[[:space:]]*;{0,1}"
+#define REGEX_CMDBYCMD "[[:space:]]*(Exec|ExecAndSet|Make|SetMode|Get|Set|Send|ExecAndSend|Timer|CancelTimer|Macro|Load|Include|SendCKPD|Emulate)[[:space:]]*(\\()|[[:space:]]*(Exit)[[:space:]]*;{0,1}"
 
 #define REGEX_SET    "[[:space:]]*(bg|editfield|filemanager|fg|font|fullscreen|icons|list|iconlist|menu|parameter|skin|status|text|title|volume|image|cover|var)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(vibrate|repaint|disconnect)[[:space:]]*"
-#define REGEX_GET    "[[:space:]]*(screen_size|cover_size|model)[[:space:]]*|[[:space:]]*(is_exists)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*"
+#define REGEX_GET    "[[:space:]]*(screen_size|cover_size|model|version|cursor)[[:space:]]*|[[:space:]]*(is_exists)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*"
 #define REGEX_TIMER  "[[:space:]]*([^[:space:]]+)[[:space:]]*,[[:space:]]*([[:digit:]]*)[[:space:]]*,[[:space:]]*([[:digit:]]*)[[:space:]]*$|[[:space:]]*([^[:space:]]+)[[:space:]]*,[[:space:]]*(cancel|pause|continue)"
 #define REGEX_MAKE   "[[:space:]]*(remote|mode|var)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*|[[:space:]]*(exit|flush|disconnect|none)[[:space:]]*"
 
-#define REGEX_SET_TL "[[:space:]]*(fg|bg|font|select)[[:space:]]*,(.*)|[[:space:]]*(add|replace)[[:space:]]*,[[:space:]]*([^[:space:]]{1}[^,]*[^[:space:]]{1})[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close)[[:space:]]*,[[:space:]]**(clear)[[:space:]]*|[[:space:]]*(clear|close|show)[[:space:]]*"
+#define REGEX_SET_TL "[[:space:]]*(fg|bg|font|select)[[:space:]]*,(.*)|[[:space:]]*(add|replace)[[:space:]]*,[[:space:]]*([^[:space:]]{1}[^,]*[^[:space:]]{1})[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close)[[:space:]]*,[[:space:]]*(clear)[[:space:]]*|[[:space:]]*(clear|close|show)[[:space:]]*"
 #define REGEX_SET_MN "[[:space:]]*(add|replace)[[:space:]]*,(.*)|[[:space:]]*(clear)[[:space:]]*"
-#define REGEX_SET_WM "[[:space:]]*(icon|window)[[:space:]]*,(.*)|[[:space:]]*(remove_all|show|close|cursor|nocursor|get_cursor|dynamic_cursor)[[:space:]]*|[[:space:]]*(set_cursor),[[:space:]]*(.*)[[:space:]]*"
+#define REGEX_SET_WM "[[:space:]]*(icon|window)[[:space:]]*,(.*)|[[:space:]]*(remove_all|show|close|cursor|nocursor|dynamic_cursor)[[:space:]]*|[[:space:]]*(set_cursor),[[:space:]]*(.*)[[:space:]]*"
 #define REGEX_SET_FM "[[:space:]]*(add|replace|select)[[:space:]]*,[[:space:]]*(left|right)[[:space:]]*,[[:space:]]*(.*)|[[:space:]]*(close|show)[[:space:]]*"
 #define REGEX_SET_VR "^[[:space:]]*([^,]*)[[:space:]]*,[[:space:]]*(.*)[[:space:]]*"
 
@@ -65,11 +66,12 @@ static char load_buf[MAXCMDLEN];
 regex_t *keyVal   = NULL;
 regex_t *cmdByCmd = NULL;
 
-mode       *modes       = NULL;
-mode       *currentMode = NULL;
-mode 	   *defMode     = NULL;
-type_alias *aliases     = NULL;
-type_key   *alarms      = NULL;
+mode       *modes        = NULL;
+mode       *currentMode  = NULL;
+mode       *internalMode = NULL;
+mode 	   *defMode      = NULL;
+type_alias *aliases      = NULL;
+type_key   *alarms       = NULL;
 
 CONF conf = { 0, 		// Log
               B19200, 		// Baudrate
@@ -85,7 +87,10 @@ CONF conf = { 0, 		// Log
 	      MODEL_DEFAULT,	// Model
 	      NULL,             // Cfg. dir
 	      -1,		// Front-end port    
-	      0};        	// Work with anyremote2html                         
+	      0,		// Work with anyremote2html
+	      "anyRemote",      // Service name
+	      0,		// set uid
+	      0};        	// set gid
 
 type_key* findExact(mode *mode, char *key)
 {
@@ -148,14 +153,43 @@ static void setCfgDir(char *d)
         	free(conf.cfgDir);
 		conf.cfgDir = NULL;
 	}
+	
+        if (getenv("AR_CFGDIR")) {	// override if set
+		conf.cfgDir = strdup(getenv("AR_CFGDIR"));
+		//printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
+		return;
+        } 
+	
+	
+	if (d != NULL) {	// search ../Utils directory
+	
+		char* try2 = malloc(strlen(d) + 10); // + "/../Utils"
+		strcpy(try2,d);
+		strcat(try2,"/../Utils");
+		
+		struct stat buf;
 
-	if (d != NULL) {
-		if (*d == '\0') {
-			conf.cfgDir = strdup(".");
-		} else {
-        		conf.cfgDir = strdup(d);
+        	if(stat(try2, &buf) == 0) {
+			try2[strlen(try2)-6] = '\0';
+			conf.cfgDir = strdup(try2);
+			
+			free(try2);
+			//printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
+			return;
 		}
-	}
+		free(try2);
+        } 
+	
+	// last resort: use {prefix}/anyremote/cfg-data directory
+        #ifdef DATADIR
+ 	conf.cfgDir = malloc(strlen(DATADIR)+20);	// + "/anyremote/cfg-data"
+        strcpy(conf.cfgDir,DATADIR);
+        strcat(conf.cfgDir,"/anyremote/cfg-data");
+	#else
+ 	conf.cfgDir = malloc(2);
+        strcpy(conf.cfgDir,".");
+        #endif
+
 	//printf("CFG DIR >%s<\n",(conf.cfgDir?conf.cfgDir:"NULL"));
 }
 
@@ -214,6 +248,12 @@ int parse_opts(int argc, char *argv[])
 			conf.http = 1;
 		} else if (strcmp(argv[i],"--autoconnect")==0 || strcmp(argv[i],"-a")==0) {
 			conf.autoConnect = 1;
+		} else if (strcmp(argv[i],"--user")==0 || strcmp(argv[i],"-u")==0) {
+			if(getUidGid(argv[++i], &conf.uid, &conf.gid)!=EXIT_OK)
+				printf("WARNING: bad username %s\n", argv[i]);
+		} else if (strcmp(argv[i],"-name")==0) {
+                        strncpy(conf.serviceName, argv[++i],MTEXTLEN-1);
+			conf.serviceName[MTEXTLEN-1] = '\0';
 		} else if (strcmp(argv[i],"-f")==0) {
 			++i;
 			continue;   // already processed this parameter
@@ -439,14 +479,6 @@ static int cmd2id (char *name)
 		return ID_MAKE;
         } else if (strcmp(name,CMD_EMU) == 0) {
 		return ID_EMU;
-        } else if (strcmp(name,CMD_REMOTEON) == 0) {
-        	printf("Command RemoteOn is deprecated. Use Make(remote,on) instead\n"); 
-        } else if (strcmp(name,CMD_REMOTEOFF) == 0) {
-        	printf("Command RemoteOff is deprecated. Use Make(remote,off) instead\n"); 
-        } else if (strcmp(name,CMD_TMPDISCONN) == 0) {
-        	printf("Command TempDisconnect is deprecated. Use Make(disconnect) instead\n"); 
-        } else if (strcmp(name,CMD_FLUSH) == 0) {
-		printf("Command Flush(...) is deprecated. Use Make(flush) instead\n");
         } 
         return ID_UNKNOWN;
 }
@@ -478,6 +510,78 @@ int cmdSet2id (const char *name)
         return ID_UNKNOWN;
 }
 
+static void addAlias(char*old, char *new)
+{
+	type_alias* Al = (type_alias*) calloc(1, sizeof(type_alias));
+
+	// Insert in head
+	if (aliases == NULL) {   // first elem
+		Al->next = NULL;
+	} else {
+		Al->next = (type_alias*) aliases;
+	}
+	aliases = Al;
+	
+	Al->key   = (char *) malloc(strlen(old)+1);
+	Al->alias = (char *) malloc(strlen(new)+1);
+
+	strcpy(Al->key,  old);
+	strcpy(Al->alias,new);
+}
+
+/* preparation for i18n
+static char* translateWord (const char *word) 
+{
+	char *trWord = (char *) malloc(strlen(word)*2+1);
+	strcpy(trWord,word);
+	strcat(trWord,word);
+	return trWord;
+}
+
+static char* translateMenu (const char *menuString) 
+{
+	//printf("trMenu %d %s\n", strlen(menuString),menuString);
+	char *bufPtr;
+	char *menuString2 = strdup(menuString);
+
+        char *token = strtok_r(menuString2,",",&bufPtr);
+	char *trString = NULL;
+	int  init = 1;
+	while (token != NULL) {
+		// translate token
+		//printf("trMenu token = %s\n", token);fflush(stdout);
+		char * trToken = translateWord(token);
+		//printf("trMenu tr token = %s\n", trToken);fflush(stdout);
+                
+		addAlias(token,trToken);
+		
+		//printf("trMenu realloc %d->%d\n", 
+		//           (trString == NULL ? 0 : strlen(trString)), 
+		//	   (trString == NULL ? 0 : strlen(trString))+strlen(trToken)+2);fflush(stdout);
+
+		trString = (char *) realloc(trString,(trString == NULL ? 0 : strlen(trString))+strlen(trToken)+2);
+		if (init) {
+			*trString = '\0';
+			strcpy(trString,trToken);
+			init = 0;
+		} else {
+			strcat(trString,trToken);
+		}	
+		
+		free(trToken);
+		token = strtok_r(NULL,",",&bufPtr);
+		
+		if (token != NULL) {
+			strcat(trString,",");
+		}
+	}
+	
+	free(menuString2);
+	//printf("trMenu %d %s\n",strlen(trString),trString);fflush(stdout);
+	return trString;
+}
+*/
+
 static int storeGetCmd(cmdItem* ci, char *cmd)
 {
 	int parseFail = 0;
@@ -921,10 +1025,28 @@ static int storeExecAndSetCmd(cmdItem* ci, char *cmd, int execFlag)
                                 			strncpy(ci->exec,start2 + result2[2].rm_so, le);
                                                 } else {
                                         		strcat (ci->descr,",");
-                                			strncat(ci->descr,start2 + result2[2].rm_so, le);
+							
+							/*
+							// attempt to use localized menu on the phone
+							char* menu = (char *) calloc(1,le+1);
+                                			strncpy(menu,start2 + result2[2].rm_so, le);
+							
+							char *trMenu = translateMenu(menu);
+							ci->descr = realloc(ci->descr,
+							               l + l2 + 3 + strlen(trMenu));
+						        //printf("REALLOC %d -> %d\n",strlen(ci->descr),
+							//               l + l2 + le2 + 2+strlen(trMenu)-le);
+                                			strcat(ci->descr,trMenu); 
+							
+							free(menu);
+							free(trMenu);
+							
+							// old non-localized variant
+							*/
+							strncat(ci->descr,start2 + result2[2].rm_so, le);
                                                 }
                                         
-                                        } else if (result2[3].rm_so >= 0) {	// clear or show|remove_all|close|cursor|nocursor|dynamic_cursor|get_cursor
+                                        } else if (result2[3].rm_so >= 0) {	// clear or show|remove_all|close|cursor|nocursor|dynamic_cursor
 
                                                 l2 = result2[3].rm_eo - result2[3].rm_so;
                                                 ci->descr = (char*) calloc(1, l + l2 + 2);
@@ -1144,6 +1266,9 @@ int storeCmds(cmdItem** cmdList, char *value)
 			start +=result->rm_eo;        
 			continue;
 		}
+		if (cmdId == ID_SETMODE ) {
+			printf("Warning: SetMode() command is deprecated !\n");
+		}
 		
 		/*if (cmds != NULL) {
 		    printf("Got command body >%s<\n",cmds);
@@ -1328,7 +1453,7 @@ static int loadKeys(FILE *fp)
 				} else {
 					nm->next = (mode*) modes;
 				}
-				modes	= nm;
+				modes = nm;
                         }
 			curMode = nm;
 	
@@ -1344,6 +1469,19 @@ static int loadKeys(FILE *fp)
 	return EXIT_OK;    
 }
 
+static void loadInternal()
+{
+	if (internalMode == NULL) {
+		internalMode = (mode*) calloc(1, sizeof(mode));
+		strcpy(internalMode->name,"_INTERNAL_");
+		internalMode->keys = NULL;
+	}
+	
+	char cmd[] = "_GET_ICON_($$)";
+	char val[] = "ExecAndSet(image,icon,echo \'UF=\"$(CfgDir)/Icons/$(Index)/$(Param).png\";echo \"$(Param),$UF\"\'|bash -f -s);";
+	storeKey(internalMode, &(internalMode->keys),cmd,val);
+}
+
 static int loadAliases(FILE *fp)
 {
         char *tmptext;
@@ -1365,19 +1503,7 @@ static int loadAliases(FILE *fp)
 		} else if(strcmp(key,SECTION_END_STR) == 0) {
 			break;
 		} else if (alias != NULL) {	// Line with key=alias definition
-			
-			type_alias* Al = (type_alias*) calloc(1, sizeof(type_alias));
-
-			// Insert in head
-			if (aliases == NULL) {   // first elem
-				Al->next = NULL;
-			} else {
-				Al->next = (type_alias*) aliases;
-			}
-			aliases = Al;
-
-			strcpy(Al->key,  key);
-			strcpy(Al->alias,alias);
+			addAlias(key,alias);
 		} else {
 			continue;
 		}
@@ -1419,15 +1545,19 @@ int load_cfg(char *mfile, int isInit)
 	//printf("LOAD:%s\n",mfile);
 
 	fp=fopen(mfile,"r");
-	if (fp == NULL) return -1;
+	if (fp == NULL) return EXIT_NOK;
 
-	keyVal  = (regex_t *) malloc(sizeof(regex_t));
-	memset(keyVal,   0, sizeof(regex_t));
-	cmdByCmd = (regex_t *) malloc(sizeof(regex_t));
-	memset(cmdByCmd, 0, sizeof(regex_t));
-
-	regexpPrepare(keyVal,   REGEX_KEYVAL);
-	regexpPrepare(cmdByCmd, REGEX_CMDBYCMD);
+        if (keyVal == NULL) {		// init it once
+		keyVal  = (regex_t *) malloc(sizeof(regex_t));
+		memset(keyVal,   0, sizeof(regex_t));
+		regexpPrepare(keyVal,   REGEX_KEYVAL);
+	}
+	
+	if (cmdByCmd == NULL) { 	// init it once
+		cmdByCmd = (regex_t *) malloc(sizeof(regex_t));
+		memset(cmdByCmd, 0, sizeof(regex_t));
+		regexpPrepare(cmdByCmd, REGEX_CMDBYCMD);
+	}
 
 	// Go through the conf file
 	while(1) {
@@ -1480,11 +1610,11 @@ int load_cfg(char *mfile, int isInit)
                         if (!isInit) {		// If we goes here from Include() command, we did all we need
                         	return EXIT_OK;
                         }
-		} else if (strcmp(tag,ALIAS_SECTION_STR) == 0/* && isInit*/) {
+		} else if (strcmp(tag,ALIAS_SECTION_STR) == 0) {
 			if (loadAliases(fp) != EXIT_OK) {
 				return EXIT_NOK;
 			}
-		} else if (strcmp(tag,ALARMS_SECTION_STR) == 0/* && isInit*/) {
+		} else if (strcmp(tag,ALARMS_SECTION_STR) == 0) {
 			if (loadAlarms(fp) != EXIT_OK) {
 				return EXIT_NOK;
 			} 
@@ -1505,7 +1635,8 @@ int load_cfg(char *mfile, int isInit)
 				strcpy(conf.cmerOff,value);
 			}	    
 		} else if (isInit) {
-			if (strncmp("GuiApp",tag,6) != 0) {	// Do not warn about GUI meta tags
+			if (strncmp("GuiApp",tag,6) != 0 &&
+			    strncmp("GuiDescription",tag,14) != 0) {	// Do not warn about GUI meta tags
 				printf("WARNING: Unknown tag in cfg file >%s<\n",tag);
 			}
 		}
@@ -1513,9 +1644,23 @@ int load_cfg(char *mfile, int isInit)
 
 	fclose(fp);
 
+	loadInternal();
+	
+
+	//printf("LOAD:%s OK\n",mfile);
 	return EXIT_OK;
 }
 
+void freeRegexps() 
+{
+	if (cmdByCmd) {
+		regfree(cmdByCmd); free(cmdByCmd); cmdByCmd = NULL;
+	}
+	if (keyVal) {
+		regfree(keyVal);   free(keyVal);   keyVal = NULL;
+	}
+}
+
 int init_cfg(char *path)
 {
 	int ret = EXIT_OK;
@@ -1535,17 +1680,16 @@ int init_cfg(char *path)
                 
 		ret = load_cfg(path,1);
 	} else {
+                setCfgDir(NULL);
+		
 		char cfgfile[MAXLEN];
-
 		strcpy(cfgfile, getenv("HOME"));
-                setCfgDir(cfgfile);
 		strcat(cfgfile, CFGFILE);
 
 		printf("Search configuration file %s\n", cfgfile);
-		if(load_cfg(cfgfile,1) != 1) {
+		if(load_cfg(cfgfile,1) != EXIT_OK) {
 			// Try again in current dir 
 			strcpy(cfgfile, ".");
-                        setCfgDir(cfgfile);
 			strcat(cfgfile, CFGFILE);
 			
 			printf("Search configuration file %s\n", cfgfile);
