@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #define _PARSE_H_ 1
 
 #include "common.h"
+#include <sys/types.h>
+#include <unistd.h>
 
 #define CFGFILE 	"/.anyremote.cfg"
 
@@ -115,11 +117,6 @@
 #define CMD_MAKE             	"Make" 
 #define CMD_EMU             	"Emulate" 
 
-#define CMD_TMPDISCONN	        "TempDisconnect"// DEPRECATED
-#define CMD_REMOTEON	        "RemoteOn"	// DEPRECATED
-#define CMD_REMOTEOFF	        "RemoteOff"	// DEPRECATED
-#define CMD_FLUSH             	"Flush" 	// DEPRECATED
-
 #define SET_BG         "bg"
 #define SET_PARAM      "parameter"
 #define SET_EFIELD     "editfield"
@@ -206,8 +203,8 @@ typedef struct Tk
 
 typedef struct Ta
 {
-	char       key   [MAXARGLEN];
-        char       alias [MAXARGLEN];
+	char       *key;
+        char       *alias;
         struct Ta  *next;
 } type_alias;
 
@@ -241,6 +238,9 @@ typedef struct
         char* cfgDir;
         int   frontEnd;
         int   http;
+        char  serviceName[MTEXTLEN];
+        uid_t   uid;
+	gid_t   gid;
 } CONF;
 
 
@@ -248,6 +248,7 @@ typedef struct
 
 char* id2Cmd            (int cmdId);
 int   init_cfg          (char *path);
+void  freeRegexps       (void);
 int   parse_opts        (int argc, char *argv[]);
 int   storeCmds         (cmdItem** cmdList, char *value);
 int   load_cfg		(char *mfile, int isInit);
