@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,12 +23,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <sys/types.h>
+#include <pwd.h>
 
 #include "common.h"
 #include "utils.h"
 #include "conf.h"
 
+
 extern char tmp[MAXMAXLEN];
+extern CONF       conf;
 
 // Some globals
 
@@ -91,7 +95,11 @@ void initLog()
                 FILE *fplog = fopen(logfile, "w");
                 if (fplog) {
                 	fclose(fplog);
-                }      
+                } 
+		if(getuid()==0 && conf.uid) { // do not open file as superuser
+			chown(logfile,conf.uid,conf.gid);
+		}
+
                 printConf();
         }
 }
@@ -100,7 +108,6 @@ void logger(char *head, char *str)
 {
 	FILE *fplog;
         time_t logtime;
-        char *timestr;
 
 	if (logfile[0] == '\0') {
         	return;
@@ -116,9 +123,13 @@ void logger(char *head, char *str)
                 if (fplog!=NULL) {
                 	if (strcmp(head, "CFG") != 0) {
                         	time(&logtime);
-                                timestr = (char *) ctime(&logtime);
-                                timestr[strlen(timestr)-1]=0;
-                                fprintf(fplog, "[%s] - ",timestr);
+				struct tm* timeinfo = localtime (&logtime );
+				
+				char stime[32];
+				stime[8] = '\0';
+				
+				sprintf(stime, "%2d:%2d:%2d", timeinfo->tm_hour, timeinfo->tm_min, timeinfo->tm_sec);
+				fprintf(fplog, "[%s] - ",stime);
                         }   
                         if(head!=NULL) {
                         	fprintf(fplog, "%s - ",head);
@@ -132,6 +143,17 @@ void logger(char *head, char *str)
         }
 }
 
+void printTime()
+{
+        time_t logtime;
+	time(&logtime);
+				
+        char *timestr = (char *) ctime(&logtime);
+        timestr[strlen(timestr)-1]=0;
+  	
+	logger("CFG", timestr);
+}	
+
 void setResFile()
 {
 	initFile(tofile, TOFILE);
@@ -433,10 +455,13 @@ extern int  flushConf;
 
 void printHelp() 
 {
-	printf("Usage: anyremote [-h] [-f conf.file] [-s connect string] [-b baudrate] [-log] [-n] [-a] [-fe port]\n\n");
+	printf("Usage: anyremote [-f conf.file] [-s connect string] [-b baudrate] [-log] [-n] [-a] \n");
+	printf("                 [-fe port] [-http] [-u username] [-name SDP service name]\n");
+	//printf("   or: anyremote -h|-v|-getconfdir\n");
+	printf("   or: anyremote -h|-v\n\n");
 	printf(" -h|--help  print this help\n");
 	printf(" -v|--version  print version of anyRemote\n");
-	printf(" -f /path/to/file  specify configutational file\n");
+	printf(" -f /path/to/file  specify configurational file\n");
 	printf(" -s connect string,  possible values\n");
 	printf("    bluetooth:<channel> (Server mode - bluetooth connection)\n");
 	printf("    socket:<port>       (Server mode - TCP/IP connection)\n");
@@ -451,9 +476,18 @@ void printHelp()
 	printf(" -n	do not send any AT+CKPD command to the phone\n");
 	printf(" -a	reconnect automatically in case of connection failure\n");
 	printf(" -fe <port> work as backend for GUI frontend. Use specified port to connect to frontend.\n");
-	printf(" -http  tune work with anyremote2html\n\n");
+	printf(" -http  tune work with anyremote2html\n");
+	printf(" -name <SDP service name> if bluetooth connection is used, allows to specify SDP service name\n");
+	//printf(" -getconfdir print path to directory with configuration files (possible to override with $AR_CFGDIR)\n");
+	printf(" -u|--user <username> if started from root, allows to set effective user ID to specified user\n\n");
+
 }
 
+//void printCfgDir() 
+//{
+//	printf("%s\n", getCfgDir()?getCfgDir():"directory with configuration files can not be determined");   
+//}
+
 int flushData() 
 {
 	//logger("DEBUG","flushData");
@@ -486,4 +520,21 @@ void flushOldConf() {
         flushAliases = NULL;
 }
 
+int getUidGid(char *username, uid_t *uid, gid_t *gid) {
+	/* Set uid and gid to the preferred user (found in setuid.h). Can either be
+	* numeric or a string, found in /etc/passwd.	*/
+	struct passwd *pw;
+	
+	if ((pw = getpwnam(username))) {
+		// Name exists
+		*uid = pw->pw_uid;
+		*gid = pw->pw_gid;
+		return EXIT_OK;
+	}
+	/* something Bad happened, so send back an error */
+	return EXIT_NOK;
+}
+
+
+
 //////////////////////////////////////////////////////////////////////////////////
