@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #define _UTILS_H
 
 #include "parse.h"
+#include <sys/types.h>
+#include <pwd.h>
 
 #define LOGFILE 	"/anyremote.log"
 
@@ -55,7 +57,8 @@ typedef struct VV
 	struct VV  *next;
 } varData;
 
-void        printHelp         (void);
+void        printHelp       (void);
+void        printTime       (void);
 void        initLog         (void);
 void        logger          (char *head, char *str);
 
@@ -78,4 +81,6 @@ void        flushOldConf    (void);
 void        setResFile      (void);
 char*       getResFile      (void);
 
+int getUidGid(char *username, uid_t *uid, gid_t *gid) ;
+
 #endif
